@@ -1,6 +1,6 @@
 #
 # ovirt-hosted-engine-setup -- ovirt hosted engine setup
-# Copyright (C) 2013-2014 Red Hat, Inc.
+# Copyright (C) 2013-2015 Red Hat, Inc.
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -22,20 +22,22 @@
 VM image creation plugin.
 """
 
-import uuid
+
 import gettext
+import uuid
 
 
-from otopi import util
 from otopi import plugin
+from otopi import util
 
 
+from ovirt_hosted_engine_ha.lib import heconflib
 from ovirt_hosted_engine_setup import constants as ohostedcons
 from ovirt_hosted_engine_setup import domains as ohosteddomains
-from ovirt_hosted_engine_setup import tasks
 
 
-_ = lambda m: gettext.dgettext(message=m, domain='ovirt-hosted-engine-setup')
+def _(m):
+    return gettext.dgettext(message=m, domain='ovirt-hosted-engine-setup')
 
 
 @util.export
@@ -193,14 +195,15 @@ class Plugin(plugin.PluginBase):
         spUUID = self.environment[ohostedcons.StorageEnv.SP_UUID]
         imgUUID = self.environment[ohostedcons.StorageEnv.IMG_UUID]
         volUUID = self.environment[ohostedcons.StorageEnv.VOL_UUID]
-        serv = self.environment[ohostedcons.VDSMEnv.VDS_CLIENT]
+        cli = self.environment[ohostedcons.VDSMEnv.VDS_CLI]
 
         if self.environment[ohostedcons.StorageEnv.DOMAIN_TYPE] in (
             ohostedcons.DomainTypes.ISCSI,
+            ohostedcons.DomainTypes.FC,
         ):
             # Checking the available space on VG where
             # we have to preallocate the image
-            vginfo = serv.s.getVGInfo(
+            vginfo = cli.getVGInfo(
                 self.environment[ohostedcons.StorageEnv.VG_UUID]
             )
             self.logger.debug(vginfo)
@@ -208,21 +211,20 @@ class Plugin(plugin.PluginBase):
                 raise RuntimeError(vginfo['status']['message'])
             vgfree = int(vginfo['info']['vgfree'])
             available_gb = vgfree / pow(2, 30)
-            if int(
-                self.environment[
-                    ohostedcons.StorageEnv.IMAGE_SIZE_GB
-                ]
-            ) > available_gb:
+            required_size = int(self.environment[
+                ohostedcons.StorageEnv.IMAGE_SIZE_GB
+            ]) + int(self.environment[
+                ohostedcons.StorageEnv.CONF_IMAGE_SIZE_GB
+            ])
+            if required_size > available_gb:
                 raise ohosteddomains.InsufficientSpaceError(
                     _(
                         'Error: the VG on block device has capacity of only '
                         '{available_gb} GiB while '
-                        '{image_gb} GiB is required for the image'
+                        '{required_size} GiB is required for the image'
                     ).format(
                         available_gb=available_gb,
-                        image_gb=self.environment[
-                            ohostedcons.StorageEnv.IMAGE_SIZE_GB
-                        ],
+                        required_size=required_size,
                     )
                 )
 
@@ -232,49 +234,26 @@ class Plugin(plugin.PluginBase):
         preallocate = ohostedcons.VolumeTypes.SPARSE_VOL
         if self.environment[ohostedcons.StorageEnv.DOMAIN_TYPE] in (
             ohostedcons.DomainTypes.ISCSI,
+            ohostedcons.DomainTypes.FC,
         ):
             # Can't use sparse volume on block devices
             preallocate = ohostedcons.VolumeTypes.PREALLOCATED_VOL
 
         diskType = 2
-        # creates a volume on the storage (SPM verb)
-        status, message = serv.createVolume([
-            sdUUID,
-            spUUID,
-            imgUUID,
-            self.environment[ohostedcons.StorageEnv.IMAGE_SIZE_GB],
+
+        heconflib.create_and_prepare_image(
+            self.logger,
+            cli,
             volFormat,
             preallocate,
-            diskType,
-            volUUID,
-            self.environment[ohostedcons.StorageEnv.IMAGE_DESC],
-        ])
-        if status == 0:
-            self.logger.debug(
-                (
-                    'Created volume {newUUID}, request was:\n'
-                    '- image: {imgUUID}\n'
-                    '- volume: {volUUID}'
-                ).format(
-                    newUUID=message,
-                    imgUUID=imgUUID,
-                    volUUID=volUUID,
-                )
-            )
-        else:
-            raise RuntimeError(message)
-        waiter = tasks.TaskWaiter(self.environment)
-        waiter.wait()
-        # Expose the image (e.g., activates the lv) on the host (HSM verb).
-        self.logger.debug('prepareImage')
-        status, message = serv.prepareImage([
-            spUUID,
             sdUUID,
+            spUUID,
             imgUUID,
             volUUID,
-        ])
-        if status != 0:
-            raise RuntimeError('Failed preparing images: %s' % message)
+            diskType,
+            self.environment[ohostedcons.StorageEnv.IMAGE_SIZE_GB],
+            self.environment[ohostedcons.StorageEnv.IMAGE_DESC],
+        )
 
 
 # vim: expandtab tabstop=4 shiftwidth=4
